@@ -1,21 +1,24 @@
 # frozen_string_literal: true
 
+# :markup: markdown
+
 require "action_dispatch/middleware/cookies"
 require "action_dispatch/middleware/flash"
 
 module ActionDispatch
   module TestProcess
     module FixtureFile
-      # Shortcut for <tt>Rack::Test::UploadedFile.new(File.join(ActionDispatch::IntegrationTest.file_fixture_path, path), type)</tt>:
+      # Shortcut for
+      # `Rack::Test::UploadedFile.new(File.join(ActionDispatch::IntegrationTest.file_fixture_path, path), type)`:
       #
-      #   post :change_avatar, params: { avatar: file_fixture_upload('david.png', 'image/png') }
+      #     post :change_avatar, params: { avatar: file_fixture_upload('david.png', 'image/png') }
       #
-      # Default fixture files location is <tt>test/fixtures/files</tt>.
+      # Default fixture files location is `test/fixtures/files`.
       #
-      # To upload binary files on Windows, pass <tt>:binary</tt> as the last parameter.
-      # This will not affect other platforms:
+      # To upload binary files on Windows, pass `:binary` as the last parameter. This
+      # will not affect other platforms:
       #
-      #   post :change_avatar, params: { avatar: file_fixture_upload('david.png', 'image/png', :binary) }
+      #     post :change_avatar, params: { avatar: file_fixture_upload('david.png', 'image/png', :binary) }
       def file_fixture_upload(path, mime_type = nil, binary = false)
         if self.class.file_fixture_path && !File.exist?(path)
           path = file_fixture(path)
@@ -30,8 +33,8 @@ module ActionDispatch
 
     def assigns(key = nil)
       raise NoMethodError,
-        "assigns has been extracted to a gem. To continue using it,
-        add `gem 'rails-controller-testing'` to your Gemfile."
+        'assigns has been extracted to a gem. To continue using it,
+        add `gem "rails-controller-testing"` to your Gemfile.'
     end
 
     def session
